@ECHO off
TITLE Image Generator

REM Initial commands if script is called from SMILES_generator.bat
IF NOT [%2]==[] (
	SET "_projectpath=%~1"
	SET "_inputfile=%_projectpath%\%~2_CAS_SMILES.txt"
	GOTO setformat
)

CLS

REM The commands listed between this point and :setformat below prepare variables
REM that contain the filenames and foldernames for the cases where this script
REM was not called by SMILES_generator.bat. 

REM If true, run initial commands for script launched by drag and drop
REM If false, run initial commands for script launched by double click
IF NOT [%1]==[] (
	SET _inputfile=%1
	ECHO Input file:
	ECHO %~1
) ELSE (
	ECHO Please enter name of input file.  Enclose your entry with double quotes ("):
	SET /P _inputfile=^>
)

FOR %%G IN (%_inputfile%) DO (
	SET "_inputfile=%%~G"
	SET "_projectpath=%%~dpG"
)

REM A choice of supported image output formats is presented. The formatting
REM options are then set based on user selection.
:setformat
ECHO.
ECHO Please choose the desired output graphics format from the following options:
ECHO.
ECHO  (E) - emf          (J) - jpeg         (M) - msbmp        (N) - png
ECHO  (P) - ppm          (S) - svg          (V) - pov
ECHO.
CHOICE /C EJMNPSV /N /M ">"
SET _CHOICE=%ERRORLEVEL%
IF %_CHOICE%==1 SET "_format=emf"    & SET "_suffix=emf"
IF %_CHOICE%==2 SET "_format=jpeg"   & SET "_suffix=jpg"
IF %_CHOICE%==3 SET "_format=msbmp"  & SET "_suffix=bmp"
IF %_CHOICE%==4 SET "_format=png"    & SET "_suffix=png"
IF %_CHOICE%==5 SET "_format=ppm"    & SET "_suffix=ppm"
IF %_CHOICE%==6 SET "_format=svg"    & SET "_suffix=svg"
IF %_CHOICE%==7 SET "_format=pov"    & SET "_suffix=pov"
IF "%_format%"=="pov" (
	SET "_format_options="
) ELSE (
	SET "_format_options=H_hetero,scale50"
)

REM A check for pre-existing output of the chosen image format is made.
IF EXIST "%_projectpath%\IMAGES\%_suffix%" (
	GOTO warning
) ELSE (
	GOTO routine
)

REM A choice is presented if the destination folder for the output of the
REM chosen image format already exists.
:warning
ECHO.
ECHO WARNING: The "%_suffix%" folder already exists. If you proceed all existing
ECHO contents of this folder will be deleted. (D)elete existing contents, choose
ECHO another (F)ormat, or (Q)uit?
CHOICE /C DFQ /N /M ">"
IF ERRORLEVEL 3 GOTO end
IF ERRORLEVEL 2 GOTO setformat
IF ERRORLEVEL 1 (
	RMDIR /s /q "%_projectpath%\IMAGES\%_suffix%"
	GOTO routine
)


:routine
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO  %_format% images
ECHO -------------------------------------------------------------------------------

MKDIR "%_projectpath%\IMAGES\%_suffix%"
TYPE NUL>"%_projectpath%\IMAGES\%_suffix%\SMILES_error"
TYPE NUL>"%_projectpath%\IMAGES\%_suffix%\name_error"


REM The first loop returns the CAS number (%%X) and SMILES string (%%Y) from
REM _inputfile using <TAB> as delimiter. The first molconvert checks %%Y is a
REM valid SMILES string. Blank or incorrect strings return an error stored in
REM SMILES_error. The results are piped to the second molconvert, which creates
REM the images. Errors for each command are stored separately. The IF statement
REM is run where at least one error file has non-zero size. Where the first
REM column of _inputfile is empty, SMILES stings in the second column will be
REM read as %%X (not %%Y). If %%X is a valid SMILES string, the user is alerted
REM to a probable missing first column entry.
SETLOCAL enabledelayedexpansion
FOR /f "usebackq tokens=1,2 delims=	" %%X IN ("%_inputfile%") DO (
	molconvert -g smiles -s "%%Y" 2>"%_projectpath%\IMAGES\%_suffix%\SMILES_error" | molconvert -g "%_format%:%_format_options%" -s "%%Y" -o "%_projectpath%\IMAGES\%_suffix%\%%~nX.%_suffix%" 2>"%_projectpath%\IMAGES\%_suffix%\name_error"
	FOR %%i IN ("%_projectpath%\IMAGES\%_suffix%\SMILES_error") DO SET _SMILES_error_size=%%~zi
	FOR %%i IN ("%_projectpath%\IMAGES\%_suffix%\name_error") DO SET _name_error_size=%%~zi
	SET /a _total_error_size=!_SMILES_error_size!+!_name_error_size!
	IF !_total_error_size! EQU 0 (
		ECHO|SET /p dummy=%%~nX.%_suffix%      
	) ELSE (
		FOR /f %%s IN ('molconvert smiles -s "%%X" 2^>NUL') DO SET _smiles_test=%%s
		ECHO.
		ECHO.
		IF "!_smiles_test!"=="" (
			ECHO  # An error occurred when generating %%~nX.%_suffix% #
		) ELSE (
			ECHO  # An output filename has not been provided for !_smiles_test! #
		)
		ECHO.
	)
)
ENDLOCAL
DEL "%_projectpath%\IMAGES\%_suffix%\SMILES_error"
DEL "%_projectpath%\IMAGES\%_suffix%\name_error"

REM The option of producing images in other formats is provided.
ECHO.
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO.
ECHO Process complete. Would you like to generate images in another format (Y/N)?
CHOICE /C YN /N /M ">"
IF ERRORLEVEL 2 GOTO end
IF ERRORLEVEL 1 GOTO setformat

:end
IF [%2]==[] (
REM Close Batch Window with exit message
	ECHO.
	ECHO Press any key to exit . . .
	PAUSE > NUL
	EXIT
)

