@ECHO off
TITLE Physicochemical property calculator

REM This section shows the parameters used in each calculation type. Changes
REM should only be made to the text on the right hand side of the '=' sign.
REM To disable any command, delete all of the text on right hand side of '=',
REM except the final quotation marks (").
SET "_ACC_COUNT=               acceptorcount --pH 7.4"
SET "_ATOM_COUNT=              atomcount"
SET "_ALIPH_COUNT=             aliphaticatomcount"
SET "_AROM_COUNT=              aromaticatomcount"
SET "_DONOR_SITE_COUNT=        donorsitecount --pH 7.4"
SET "_CHARGE=                  formalcharge --pH 7.4"
SET "_FORMULA=                 formula"
SET "_MASS=                    mass"
SET "_IUPAC_NAME=              name -e ..UTF-8"
SET "_LOGD=                    logd --lower 1.4 --upper 13.4 --step 1 --weights 1:1:1:0 --considertautomerization"
SET "_PSA=                     polarsurfacearea --pH 7.4"
SET "_ROTATABLE=               rotatablebondcount"
SET "_SORT_FORMULA=            sortableformula -d 3"
SET "_SORT_FORMULA_MAJOR_FRAG= sortableformula -d 3 --single true"

REM Initial commands if the script was called from SMILES_generator.bat
IF NOT [%2]==[] (
	SET "_projectpath=%~1"
	SET "_inputfile=%_projectpath%\%~2_SMILES.txt"
	SET _prefix=%~2
	GOTO routine
)
CLS

REM The commands listed between this point and :routine below prepare variables
REM that contain the filenames and foldernames for the cases where this script
REM was not called by SMILES_generator.bat. 

REM If true, run initial commands for script launched by drag and drop
REM If false, run initial commands for script launched by double click
IF NOT [%1]==[] (
	SET "_inputfile=%1"
	ECHO Input file:
	ECHO %~1
	ECHO.
) ELSE (
	ECHO Please enter name of input file. Enclose your entry with double quotes ^("):
	SET /P _inputfile=^>
)
ECHO.

REM The project directory is taken as the parent directory of the input file.
FOR %%G IN (%_inputfile%) DO (
	SET "_inputfile=%%~G"
	SET "_inputfilename=%%~nG"
	SET "_projectpath=%%~dpG"
)
REM Checks to see if "_SMILES" is already appended to the input filename to
REM avoid duplication in the names of output files.
IF "dummy%_inputfilename:_SMILES=%"=="dummy%_inputfilename%" (
	SET "_prefix=%_inputfilename%"
	) ELSE (
	SET "_prefix=%_inputfilename:_SMILES=%"
)

REM Checks if the CALCS folder already exists in the project directory.
IF EXIST "%_projectpath%\CALCS" (
	GOTO warning
) ELSE (
	GOTO routine
)

REM A choice is presented when the target CALCS folder already exists.
:warning
ECHO WARNING: The CALCS folder already exists. If you proceed all existing contents
ECHO of this folder will be deleted. (D)elete folder contents or (Q)uit?
CHOICE /C DQ /N /M ">"
IF ERRORLEVEL 2 GOTO end
IF ERRORLEVEL 1 (
	RMDIR /s /q "%_projectpath%\CALCS"
	GOTO routine
)

:routine
MKDIR "%_projectpath%\CALCS"

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO  Physico-chemical property calculations
ECHO -------------------------------------------------------------------------------
ECHO.
ECHO|SET /p dummy=Please wait. This may take several minutes

REM Temporary error file is created.
TYPE NUL>"%_projectpath%\CALCS\temp_error"

REM %_FORMULA% and %_SORT_FORMULA% calculates chemical formula for all
REM fragments i.e. including small fragments like counter ions, whereas
REM %_SORT_FORMULA_MAJOR_FRAG% calculates the formula for only the largest
REM fragment. The molconvert command should do nothing, but is included as the
REM use of the pipe afterwards has the desired effect of preserving empty lines
REM present in the input file.
molconvert -g smiles "%_inputfile%" 2>>"%_projectpath%\CALCS\temp_error" | cxcalc -i SMILES %_FORMULA% %_SORT_FORMULA% %_SORT_FORMULA_MAJOR_FRAG% >"%_projectpath%\CALCS\%_prefix%_FORMULAS.txt" 2>NUL
ECHO|SET /p dummy=.

REM Generates an IUPAC name for each compound, including all fragments.
ECHO IUPAC NAME>"%_projectpath%\CALCS\%_prefix%_IUPAC_NAME.txt"
molconvert -g smiles "%_inputfile%" 2>>"%_projectpath%\CALCS\temp_error" | molconvert %_IUPAC_NAME% >>"%_projectpath%\CALCS\%_prefix%_IUPAC_NAME.txt" 2>NUL
ECHO|SET /p dummy=.

REM molconvert extracts the largest molecular fragment then cxcalc calculates
REM various properties for the largest fragment i.e. small fragments (like
REM counter ions) are excluded when performing these calculations.
molconvert -g smiles -F "%_inputfile%" 2>>"%_projectpath%\CALCS\temp_error" | cxcalc -i SMILES %_MASS% %_ATOM_COUNT% %_ALIPH_COUNT% %_AROM_COUNT% %_ROTATABLE% %_ACC_COUNT% %_DONOR_SITE_COUNT% %_PSA% %_CHARGE% %_LOGD% >"%_projectpath%\CALCS\%_prefix%_LARGEST_FRAGMENT_ONLY.txt" 2>NUL
ECHO|SET /p dummy=.done 

REM If any errors were generated, a warning is displayed.
FOR %%i IN ("%_projectpath%\CALCS\temp_error") DO SET _filesize=%%~zi
IF %_filesize% NEQ 0 (
	ECHO. 
	ECHO.
	ECHO AN ERROR OCCURRED WHEN PROCESSING ONE OR MORE COMPOUNDS!! Please ensure that 
	ECHO the input file contains valid SMILES stings. To re-run, drag and drop the
	ECHO amended input file onto the Physicochem_calculator.bat icon.
)
ECHO.

REM Temporary error file is removed.
DEL "%_projectpath%\CALCS\temp_error"

REM Where this script was called by SMILES_generator.bat, excecution of 
REM SMILES_generator.bat now resumes. Otherwise the window closes.
:end
IF [%2]==[] (
	ECHO.
	ECHO Press any key to exit . . .
	PAUSE > NUL
	EXIT
)
