@ECHO off
CLS
TITLE SMILES generator

REM If true, run initial commands for script launched by drag and drop
REM If false, run initial commands for script launched by double click
IF NOT [%1]==[] (
	SET "_inputfile=%1"
	ECHO Input file:
	ECHO %1
) ELSE (
	ECHO Please enter name of input file containing CAS Numbers ^(one per line^). Enclose your entry with double quotes ^("):
	SET /P "_inputfile=>" 
)
ECHO. 

REM The user is queried for a name for the project folder
:foldername
ECHO Please enter folder name for output files:
SET /P "_foldername=>"
FOR %%G IN (%_inputfile%) DO SET "_projectpath=%%~dpG%_foldername%"
ECHO. 

IF EXIST "%_projectpath%" GOTO warning
MKDIR "%_projectpath%"
GOTO routine

REM A choice is presented when project folder of the same name and path already exists
:warning
ECHO WARNING: The folder named "%_foldername%" already exists.
ECHO If you proceed all existing contents of this folder will be deleted.
ECHO (D)elete contents, (E)nter a different foldername, or (Q)uit?
CHOICE /C DEQ /N /M ">"
IF ERRORLEVEL 3 GOTO end
IF ERRORLEVEL 2 ECHO. & GOTO foldername
IF ERRORLEVEL 1 (
	RMDIR /s /q "%_projectpath%"
	MKDIR "%_projectpath%"
	ECHO.
	GOTO routine
)

:routine
REM Double quotes are removed from the path of the inputfile.
FOR %%G IN (%_inputfile%) DO SET "_inputfile=%%~G"

ECHO|SET /p dummy=Generating SMILES

REM The following takes the _inputfile and converts it to ASCII by printing it
REM to temp_input. The FOR loop adds a <TAB> to blank lines in temp_input and
REM stores the result in tabbed_temp_input. temp_input is then removed.
TYPE "%_inputfile%">"%_projectpath%\temp_input"
TYPE NUL>"%_projectpath%\tabbed_temp_input"
SETLOCAL enabledelayedexpansion
FOR /f %%N IN ('FIND /c /v "" ^<"%_projectpath%\temp_input"') DO SET count=%%N
<"%_projectpath%\temp_input" (
	FOR /l %%N IN (1 1 %count%) DO (
		SET "ln="
		SET /p "ln="
		IF "!ln!"=="" (
			ECHO:	>>"%_projectpath%\tabbed_temp_input"
		) ELSE (
			ECHO !ln!>>"%_projectpath%\tabbed_temp_input"
		)
	)
)
ENDLOCAL
DEL "%_projectpath%\temp_input"

TYPE NUL>"%_projectpath%\%_foldername%_SMILES.txt"
TYPE NUL>"%_projectpath%\%_foldername%_CAS_SMILES.txt"
TYPE NUL>"%_projectpath%\temp_error"
TYPE NUL>"%_projectpath%\temp_output"
SET _firstline=

REM The first loop returns all lines of tabbed_temp_input, including those that
REM were blank in _inputfile. The second loop processes each line and both 
REM STDOUT and STDERR of molconvert are stored in temp_output. ECHO dummy
REM (which always produces STDOUT), ensures that the DO loop is executed when
REM molconvert only produces STDERR. STDERR from molconvert can be multiline
REM and always contains spaces. The top line of molconvert output is stored as
REM _firstline. The IF statement tests _firstline for spaces to decide if the
REM result is erroneous or valid, then prints appropriate output to the various
REM output files. All errors are stored in temp_error.
SETLOCAL enabledelayedexpansion
FOR /f "usebackq delims=" %%X IN ("%_projectpath%\tabbed_temp_input") DO (
	FOR /f "delims=" %%i IN ('molconvert -g smiles -s "%%X" ^>"%_projectpath%\temp_output" 2^>^&1 ^& ECHO dummy') DO (
		SET /p _firstline=<"%_projectpath%\temp_output" 
		IF "dummy!_firstline: =!"=="dummy!_firstline!" (
			ECHO !_firstline!>>                "%_projectpath%\%_foldername%_SMILES.txt"
			ECHO %%X	!_firstline!>>         "%_projectpath%\%_foldername%_CAS_SMILES.txt"
			ECHO|SET /p dummy=.
		) ELSE (
			ECHO.>>                            "%_projectpath%\temp_error"
			TYPE "%_projectpath%\temp_output">>"%_projectpath%\temp_error"
			ECHO.>>                            "%_projectpath%\temp_error"
			ECHO.>>                            "%_projectpath%\%_foldername%_SMILES.txt"
			IF "%%X"=="	" (
				ECHO.>>                        "%_projectpath%\%_foldername%_CAS_SMILES.txt"
			) ELSE (
				ECHO %%X>>                     "%_projectpath%\%_foldername%_CAS_SMILES.txt"
			)
			ECHO|SET /p dummy=X
		)
	)
)
ENDLOCAL
DEL "%_projectpath%\tabbed_temp_input"
DEL "%_projectpath%\temp_output"

ECHO done
ECHO.
ECHO %_foldername%_SMILES.txt created.
ECHO %_foldername%_CAS_SMILES.txt created.

REM Where temp_error has a non-zero filesize, an error has occurred.
FOR %%i IN ("%_projectpath%\temp_error") DO SET _filesize=%%~zi
IF %_filesize% EQU 0 (GOTO calcs) ELSE (GOTO error)

REM Errors are displayed on-screen and a choice is offered whether or not to
REM manually insert the missing SMILES values. If yes, the files are opened in
REM notepad for editing, whereupon the user enters his SMILES strings, saves
REM and then can continue. If no, the images and physicochemical data will not
REM be produced for CAS numbers that could not be converted.
:error
ECHO. 
ECHO WARNING: Error(s) have occurred when processing one or more entries in the
ECHO input file. No SMILES strings were generated for these entries.
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO  Error details
ECHO -------------------------------------------------------------------------------
TYPE "%_projectpath%\temp_error"
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO.
ECHO Do you want to manually input the missing SMILES before continuing (Y/N)?
CHOICE /C YN /N /M ">"
IF ERRORLEVEL 2 GOTO calcs
IF ERRORLEVEL 1 (
	ECHO.
	ECHO The files that need to be updated will be opened in notepad. Press any key to
	ECHO continue . . .
	PAUSE>NUL
	START %windir%\System32\Notepad.exe "%_projectpath%\%_foldername%_SMILES.txt"
	START %windir%\System32\Notepad.exe "%_projectpath%\%_foldername%_CAS_SMILES.txt"
	ECHO.
	ECHO When finished editing, ensure that you save both "%_foldername%_SMILES.txt" and "%_foldername%_CAS_SMILES.txt". Press any key to continue . . .
	PAUSE>NUL
)

REM If desired, Physicochem_Calculator.bat is called with %_foldername%_SMILES.txt as input.
:calcs
DEL "%_projectpath%\temp_error"
ECHO.
ECHO Do you want to use "%_foldername%_SMILES.txt" to calculate physico-chemical properties (Y/N)?
CHOICE /C YN /N /M ">"
IF ERRORLEVEL 2 GOTO images
IF ERRORLEVEL 1 CALL "%~dp0Physicochem_Calculator.bat" "%_projectpath%" "%_foldername%"

REM If desired, Image_generator.bat is called with %_foldername%_CAS_SMILES.txt as input.
:images
ECHO.
ECHO Do you want to use "%_foldername%_CAS_SMILES.txt" to generate PNG and SVG images (Y/N)?
CHOICE /C YN /N /M ">"
IF ERRORLEVEL 2 GOTO end
IF ERRORLEVEL 1 CALL "%~dp0Image_generator.bat" "%_projectpath%" "%_foldername%"

:end
ECHO.
ECHO Press any key to exit . . .
PAUSE > NUL
EXIT