@ECHO OFF
TITLE Individual Physico-Chemical Property Calculator

CLS

REM This section shows the parameters used in each calculation type. Changes
REM should only be made to the text on the right hand side of the '=' sign.
REM To disable any command, delete all of the text on right hand side of '=',
REM except the final quotation marks (").
SET "_ACC_COUNT_params=               acceptorcount --pH 7.4"
SET "_ATOM_COUNT_params=              atomcount"
SET "_ALIPH_COUNT_params=             aliphaticatomcount"
SET "_AROM_COUNT_params=              aromaticatomcount"
SET "_DONOR_SITE_COUNT_params=        donorsitecount --pH 7.4"
SET "_CHARGE_params=                  formalcharge --pH 7.4"
SET "_FORMULA_params=                 formula"
SET "_MASS_params=                    mass --precision 2"
SET "_PSA_params=                     polarsurfacearea --pH 7.4"
SET "_ROTATABLE_params=               rotatablebondcount"
SET "_SORT_FORMULA_params=            sortableformula -d 3"
SET "_CXCALC_IUPAC_NAME_params=       name"
SET "_OLD_NAME_params=                name --type traditional"
SET "_LOGD_params=                    logd --lower 1.4 --upper 13.4 --step 1 --weights 1:1:1:0 --considertautomerization"
SET "_IUPAC_NAME_params=              name -e ..UTF-8"
SET "_CAS_RN_params=                  name:cas#"


IF NOT [%1]==[] SET "_INPUT=%1"

:start
ECHO.
IF "%_INPUT%"=="" (
	ECHO Please enter a compound string  or type 'format' for a list of valid formats
	SET /p "_INPUT=>"
) ELSE (
	ECHO Your input file is %_INPUT%
	SET "_STRING= "
)

IF EXIST %_INPUT% (
	SET "_STRING= "
) ELSE (
	SET "_STRING=-s"
)

IF "%_INPUT%"=="format" (
	CLS
	ECHO Ŀ
	ECHO                              Valid Input Formats                             
	ECHO 
	ECHO.
	ECHO  There are two main ways to submit molecules to this script. Firstly, at the
	ECHO  prompt you may enter ^(or copy and paste^) single line format strings such as
	ECHO  SMILES, CAS RN, a common name, an IUPAC name or a peptide string ^(all are
	ECHO  case-sensitive^).
	ECHO.
	ECHO  The second way is to drag and drop files that contain a single molecule either
	ECHO  onto the single_compound_calculator.bat icon, which will launch the script, or
	ECHO  into the open batch window at the prompt. The file extension is not important,
	ECHO  but the file contents must be in one of the following formats: mol, rgf, sdf, 
	ECHO  rdf, csmol, csrgf, cssdf, csrdf, cml, cxsmiles, abbrevgroup, sybyl, mol2, pdb,
	ECHO  xyz, inchi, SMILES, CAS RN, common names, IUPAC name or a peptide string.
	ECHO.
	ECHO 
	ECHO.
	SET _INPUT=
	PAUSE
	GOTO start
)

TYPE NUL>temp_error
SET _SMILES_AF_val=
FOR /f %%A IN ('molconvert smiles %_STRING% "%_INPUT%" 2^>^>temp_error') DO SET "_SMILES_AF_val=%%A"
FOR %%i IN (temp_error) DO SET _filesize=%%~zi
IF NOT %_filesize% EQU 0 GOTO error
DEL temp_error

SET _SMILES_MF_val=
FOR /f %%A IN ('molconvert smiles -F %_STRING% "%_INPUT%"') DO SET "_SMILES_MF_val=%%A"

IF "%_SMILES_AF_val%"=="%_SMILES_MF_val%" (
	SET _FRAGMENTS=FALSE
) ELSE (
	SET _FRAGMENTS=TRUE
)

ECHO.
ECHO Ŀ
IF %_FRAGMENTS%==FALSE (
ECHO                          Physico-chemical Properties                         
) ELSE (
ECHO             Physico-chemical Properties ^(All compound fragments^)             
)
ECHO 

call cxcalc -N i "%_SMILES_AF_val%" %_ACC_COUNT_params% %_ATOM_COUNT_params% %_ALIPH_COUNT_params% %_AROM_COUNT_params% %_DONOR_SITE_COUNT_params% %_CHARGE_params% %_FORMULA_params% %_MASS_params% %_PSA_params% %_ROTATABLE_params% %_SORT_FORMULA_params% %_CXCALC_IUPAC_NAME_params% %_OLD_NAME_params% %_LOGD_params% >temp_properties

SET _IUPAC_NAME_AF_val=
FOR /f "delims=" %%A IN ('molconvert %_IUPAC_NAME_params% %_STRING% "%_SMILES_AF_val%" 2^>NUL') DO SET "_IUPAC_NAME_AF_val=%%A"
IF "%_IUPAC_NAME_AF_val%"=="" SET _IUPAC_NAME_AF_val=Unknown

SET _CAS_RN_AF_val=
FOR /f "delims=" %%A IN ('molconvert %_CAS_RN_params% %_STRING% "%_SMILES_AF_val%" 2^>NUL') DO SET "_CAS_RN_AF_val=%%A"

IF "%_CAS_RN_AF_val%"=="" (
	SET _CAS_RN_AF_val=Unknown
) ELSE (
	SET "_CAS_RN_AF_val=%_CAS_RN_AF_val:,=  %"
)

FOR /f "skip=1 tokens=1-26 delims=	" %%A IN (temp_properties) DO (
	SET "_ACC_COUNT_AF_val=%%A"
	SET "_ATOM_COUNT_AF_val=%%B"
	SET "_ALIPH_COUNT_AF_val=%%C"
	SET "_AROM_COUNT_AF_val=%%D"
	SET "_DONOR_SITE_COUNT_AF_val=%%E"
	SET "_CHARGE_AF_val=%%F"
	SET "_FORMULA_AF_val=%%G"
	SET "_MASS_AF_val=%%H"
	SET "_PSA_AF_val=%%I"
	SET "_ROTATABLE_AF_val=%%J"
	SET "_SORT_FORMULA_AF_val=%%K"
	SET "_CXCALC_IUPAC_NAME_AF_val=%%L"
	SET "_OLD_NAME_AF_val=%%M"
	SET "_LOGD_AF_val=%%T"
	SET "_ALL_LOGD_AF_val=%%N	%%O	%%P	%%Q	%%R	%%S	%%T	%%U	%%V	%%W	%%X	%%Y	%%Z"
)

IF %_FRAGMENTS%==TRUE (
	ECHO    Molar mass:                   %_MASS_AF_val% Da
	ECHO  * Chemical Formula:             %_FORMULA_AF_val%
	ECHO  * Sortable Formula:             %_SORT_FORMULA_AF_val%
	ECHO.
	ECHO    No. of atoms:                 %_ATOM_COUNT_AF_val%
	ECHO    No. of aliphatic atoms:       %_ALIPH_COUNT_AF_val%
	ECHO    No. of aromatic atoms:        %_AROM_COUNT_AF_val%
	ECHO    No. of rotatable bonds:       %_ROTATABLE_AF_val%
) ELSE (
	ECHO  * Molar mass:                   %_MASS_AF_val% Da
	ECHO  * Chemical Formula:             %_FORMULA_AF_val%
	ECHO  * Sortable Formula:             %_SORT_FORMULA_AF_val%
	ECHO.
	ECHO  * No. of atoms:                 %_ATOM_COUNT_AF_val%
	ECHO  * No. of aliphatic atoms:       %_ALIPH_COUNT_AF_val%
	ECHO  * No. of aromatic atoms:        %_AROM_COUNT_AF_val%
	ECHO  * No. of rotatable bonds:       %_ROTATABLE_AF_val%
)
IF %_FRAGMENTS%==FALSE (
	ECHO.
	ECHO  * No. of H-bond acceptor atoms: %_ACC_COUNT_AF_val%
	ECHO  * No. of H-bond donor atoms:    %_DONOR_SITE_COUNT_AF_val%
	ECHO  * Polar surface area ^(PSA^):     %_PSA_AF_val% 
	
	IF NOT %_CHARGE_AF_val:-=%==%_CHARGE_AF_val% (
		ECHO  * Overall charge at pH 7.4:     %_CHARGE_AF_val%
	) ELSE (
		IF %_CHARGE_AF_val%==0 (
			ECHO  * Overall charge at pH 7.4:     %_CHARGE_AF_val%
		) ELSE (
			ECHO  * Overall charge at pH 7.4:     +%_CHARGE_AF_val%
		)
	)
	ECHO.
	ECHO  * Log D at pH 7.4:              %_LOGD_AF_val%
)
ECHO.
ECHO  * CAS RN:           %_CAS_RN_AF_val%
ECHO  * SMILES:           %_SMILES_AF_val%

SET "_OLD_NAME_AF_TEST=%_OLD_NAME_AF_val:(=%"
SET "_OLD_NAME_AF_TEST=%_OLD_NAME_AF_TEST:)=%"

IF "%_OLD_NAME_AF_val%"=="%_OLD_NAME_AF_TEST%" GOTO NoBrackets
IF NOT "%_CXCALC_IUPAC_NAME_AF_val%"=="%_OLD_NAME_AF_val%" ECHO  * Traditional Name: "%_OLD_NAME_AF_val%"
GOTO PostNameCheck

:NoBrackets
IF NOT "%_CXCALC_IUPAC_NAME_AF_val%"=="%_OLD_NAME_AF_val%" ECHO  * Traditional Name: %_OLD_NAME_AF_val%

:PostNameCheck
IF "_OLD_NAME_AF_val"=="" SET "_OLD_NAME_AF_val=Unknown"
ECHO  * IUPAC Name:       %_IUPAC_NAME_AF_val%
IF %_FRAGMENTS%==FALSE GOTO end

ECHO.
ECHO Ŀ
ECHO          Physico-chemical Properties (Major compound fragment only)          
ECHO 

call cxcalc -N i "%_SMILES_MF_val%" %_ACC_COUNT_params% %_ATOM_COUNT_params% %_ALIPH_COUNT_params% %_AROM_COUNT_params% %_DONOR_SITE_COUNT_params% %_CHARGE_params% %_FORMULA_params% %_MASS_params% %_PSA_params% %_ROTATABLE_params% %_SORT_FORMULA_params% %_CXCALC_IUPAC_NAME_params% %_OLD_NAME_params% %_LOGD_params% >temp_properties

SET _IUPAC_NAME_MF_val=
FOR /f "delims=" %%A IN ('molconvert %_IUPAC_NAME_params% %_STRING% "%_SMILES_MF_val%" 2^>NUL') DO SET "_IUPAC_NAME_MF_val=%%A"
IF "%_IUPAC_NAME_MF_val%"=="" SET _IUPAC_NAME_MF_val=Unknown

SET _CAS_RN_MF_val=
FOR /f "delims=" %%A IN ('molconvert %_CAS_RN_params% %_STRING% "%_SMILES_MF_val%" 2^>NUL') DO SET "_CAS_RN_MF_val=%%A"

IF "%_CAS_RN_MF_val%"=="" (
	SET _CAS_RN_MF_val=Unknown
) ELSE (
	SET "_CAS_RN_MF_val=%_CAS_RN_MF_val:,=; %"
)

FOR /f "skip=1 tokens=1-26 delims=	" %%A IN (temp_properties) DO (
	SET "_ACC_COUNT_MF_val=%%A"
	SET "_ATOM_COUNT_MF_val=%%B"
	SET "_ALIPH_COUNT_MF_val=%%C"
	SET "_AROM_COUNT_MF_val=%%D"
	SET "_DONOR_SITE_COUNT_MF_val=%%E"
	SET "_CHARGE_MF_val=%%F"
	SET "_FORMULA_MF_val=%%G"
	SET "_MASS_MF_val=%%H"
	SET "_PSA_MF_val=%%I"
	SET "_ROTATABLE_MF_val=%%J"
	SET "_SORT_FORMULA_MF_val=%%K"
	SET "_CXCALC_IUPAC_NAME_MF_val=%%L"
	SET "_OLD_NAME_MF_val=%%M"
	SET "_LOGD_MF_val=%%T"
	SET "_ALL_LOGD_MF_val=%%N	%%O	%%P	%%Q	%%R	%%S	%%T	%%U	%%V	%%W	%%X	%%Y	%%Z"
)

ECHO  * Molar mass:                   %_MASS_MF_val% Da
ECHO    Chemical Formula:             %_FORMULA_MF_val%
ECHO  * Sortable Formula:             %_SORT_FORMULA_MF_val%
ECHO.
ECHO  * No. of atoms:                 %_ATOM_COUNT_MF_val%
ECHO  * No. of aliphatic atoms:       %_ALIPH_COUNT_MF_val%
ECHO  * No. of aromatic atoms:        %_AROM_COUNT_MF_val%
ECHO  * No. of rotatable bonds:       %_ROTATABLE_MF_val%
ECHO.
ECHO  * No. of H-bond acceptor atoms: %_ACC_COUNT_MF_val%
ECHO  * No. of H-bond donor atoms:    %_DONOR_SITE_COUNT_MF_val%
ECHO  * Polar surface area (PSA):     %_PSA_MF_val% 
IF NOT %_CHARGE_MF_val:-=%==%_CHARGE_MF_val% (
	ECHO  * Overall charge at pH 7.4:     %_CHARGE_MF_val%
) ELSE (
	IF %_CHARGE_MF_val%==0 (
		ECHO  * Overall charge at pH 7.4:     %_CHARGE_MF_val%
	) ELSE (
		ECHO  * Overall charge at pH 7.4:     +%_CHARGE_MF_val%
	)
)
ECHO.
ECHO  * Log D at pH 7.4:              %_LOGD_MF_val%
ECHO.
ECHO    CAS RN:           %_CAS_RN_MF_val%
ECHO    SMILES:           %_SMILES_MF_val%

SET "_OLD_NAME_MF_TEST=%_OLD_NAME_MF_val:(=%"
SET "_OLD_NAME_MF_TEST=%_OLD_NAME_MF_TEST:)=%"

IF "%_OLD_NAME_MF_val%"=="%_OLD_NAME_MF_TEST%" GOTO NoBracketsM
IF "%_CXCALC_IUPAC_NAME_MF_val%"=="%_OLD_NAME_MF_val%" ECHO  * Traditional Name: "%_OLD_NAME_MF_val%"
GOTO PostNameCheckM

:NoBracketsM
IF NOT "%_CXCALC_IUPAC_NAME_MF_val%"=="%_OLD_NAME_MF_val%" ECHO  * Traditional Name: %_OLD_NAME_MF_val%

:PostNameCheckM
IF "_OLD_NAME_MF_TEST"=="" SET "_OLD_NAME_MF_val=Unknown"
ECHO  * IUPAC Name:       %_IUPAC_NAME_MF_val%

:end
ECHO 
CHOICE /M "Send asterisked output to text file"
IF ERRORLEVEL 2 GOTO ImageProduction
SET /p _HEADER=< temp_properties
SET "_LOGD_HEADER=%_HEADER:~190%"
ECHO Traditional Name	CAS RN			SMILES	IUPAC Name	Chemical Formula	Sortable Formula	Sortable Formula ^(Major Fragment only^)														Molar Mass	Atom Count	Aliphatic Atom Count	Aromatic Atom Count		Rotatable Bond Count	H-Bond Acceptors	H-Bond Donors	TPSA	Formal Charge	%_LOGD_HEADER%>properties.txt
	
FOR /f "tokens=1 delims= " %%A IN ('ECHO %_CAS_RN_AF_val%') DO SET "_CAS_RN_AF_val=%%A"
	
IF %_FRAGMENTS%==TRUE ( GOTO FragmentedMolecules ) ELSE ( GOTO NonFragmentedMolecules )

:FragmentedMolecules
ECHO %_OLD_NAME_AF_val%	%_CAS_RN_AF_val%			%_SMILES_AF_val%	%_IUPAC_NAME_AF_val%	%_FORMULA_AF_val%	%_SORT_FORMULA_AF_val%	%_SORT_FORMULA_MF_val%														%_MASS_MF_val%	%_ATOM_COUNT_MF_val%	%_ALIPH_COUNT_MF_val%	%_AROM_COUNT_MF_val%		%_ROTATABLE_MF_val%	%_ACC_COUNT_MF_val%	%_DONOR_SITE_COUNT_MF_val%	%_PSA_MF_val%	%_CHARGE_MF_val%	%_ALL_LOGD_MF_val%>>properties.txt
GOTO OpenOutput

:NonFragmentedMolecules
ECHO %_OLD_NAME_AF_val%	%_CAS_RN_AF_val%			%_SMILES_AF_val%	%_IUPAC_NAME_AF_val%	%_FORMULA_AF_val%	%_SORT_FORMULA_AF_val%	%_SORT_FORMULA_AF_val%														%_MASS_AF_val%	%_ATOM_COUNT_AF_val%	%_ALIPH_COUNT_AF_val%	%_AROM_COUNT_AF_val%		%_ROTATABLE_AF_val%	%_ACC_COUNT_AF_val%	%_DONOR_SITE_COUNT_AF_val%	%_PSA_AF_val%	%_CHARGE_AF_val%	%_ALL_LOGD_AF_val%>>properties.txt

:OpenOutput
START %windir%\System32\Notepad.exe properties.txt

:ImageProduction
DEL temp_properties
CHOICE /M "Create png and svg images for this compound"
IF ERRORLEVEL 2 GOTO repeat
CALL molconvert "png:H_hetero,scale50" %_STRING% "%_SMILES_AF_val%" -o "temp_png_image.png"
temp_png_image.png
CALL molconvert "svg:H_hetero,scale50" %_STRING% "%_SMILES_AF_val%" -o "temp_svg_image.svg"

:repeat
CHOICE /M "Calculate properties for another molecule"
IF ERRORLEVEL 2 ECHO Press any key to exit . . . & PAUSE>NUL & EXIT
IF ERRORLEVEL 1 (
	SET "_INPUT="
	CLS
	GOTO start
)


:error
TYPE temp_error
DEL temp_error
SET _INPUT=
ECHO.
PAUSE
CLS
GOTO start